from . import basiciv
from . import worksiv


class User(basiciv.BasicConfig):

    def __init__(self, author_id: int or str = None, **kwargs):
        super(User, self).__init__(**kwargs)
        self.author_id = author_id
        self.following_all = basiciv.Queue()

        if author_id:
            person = 'https://www.pixiv.net/ajax/user/%s?' \
                     'full=1' % author_id
            profile = 'https://www.pixiv.net/ajax/user/%s/profile/' \
                      'all' % author_id

            self.sess.headers['Referer'] = \
                'https://www.pixiv.net/member.php?' \
                'id=%s' % author_id

            person_req = self.sess.get(
                person, headers=self.sess.headers, timeout=self.kvpair['timeout']
            )
            self.person_interface = person_req.json()

            if self.person_interface['error']:
                raise basiciv.exceptions.AjaxRequestError(
                    self.person_interface['message']
                )

            self.author_name = self.person_interface['body']['name']
            self.following_total = self.person_interface['body']['following']
            self.social = self.person_interface['body']['social']
            self.premium = self.person_interface['body']['premium']

            profile_request = self.sess.get(
                profile, headers=self.sess.headers, timeout=self.kvpair['timeout']
            )
            self.profile_interface = profile_request.json()

            if self.profile_interface['error']:
                raise basiciv.exceptions.AjaxRequestError(
                    self.profile_interface['message']
                )

            if 'illusts' in self.profile_interface['body']:
                self.illusts = basiciv.Queue([
                    worksiv.Works(illust_id=illust_id) for illust_id in list(
                        self.profile_interface['body']['illusts'].keys()
                    )])

            if 'manga' in self.profile_interface['body']:
                self.mangas = basiciv.Queue([
                    worksiv.Works(illust_id=illust_id) for illust_id in list(
                        self.profile_interface['body']['manga'].keys()
                    )])

            self.sess.headers['Referer'] = \
                'https://www.pixiv.net/bookmark.php?' \
                'id=%s&type=user' % self.author_id

            offset = 0
            while offset < self.following_total:
                following = 'https://www.pixiv.net/ajax/user/%s/following?' \
                            'offset=%s&limit=20&rest=show' % (
                                self.author_id, offset)

                follow_request = self.sess.get(
                    following, headers=self.sess.headers, timeout=self.kvpair['timeout']
                )
                follow_interface = follow_request.json()

                if follow_interface['error']:
                    raise basiciv.exceptions.AjaxRequestError(
                        follow_interface['message']
                    )

                for users_iter in follow_interface['body']['users']:
                    self.following_all.que_tar.append(users_iter)
                offset += 20

    def details(self):
        return self.person_interface['body']

    def run(self, author_id: int or str or None):
        self.__init__(author_id)

    def bookmark(self) -> True or None:
        if 'Cookie' not in self.sess.headers:
            raise basiciv.exceptions.PixivError('You must be logged in before using this feature!')

        interface = self.sess.post(
            'https://www.pixiv.net/bookmark_add.php',
            data={
                'mode': 'add',
                'type': 'user',
                'user_id': self.author_id,
                'tags': '',
                'restrict': 0,
                'format': 'json',
            },
            headers={
                'x-csrf-token': self.token
            }
        ).json()
        if not interface['error']:
            return True
        else:
            raise basiciv.exceptions.AutheVerifyError(interface['message'])



from requests.compat import unquote_plus
from . import basiciv
from PIL import Image
from io import BytesIO


class WorksTag(basiciv.BasicConfig):

    def __init__(self, tag: int or str = None, **kwargs):
        super(WorksTag, self).__init__(**kwargs)
        if tag:
            self.url = 'https://www.pixiv.net/ajax/tag/' \
                       '%s/interface' % tag
            self.im_data = None

            r = self.sess.get(
                self.url,
                headers=self.sess.headers,
                timeout=5
            )
            self.interface = r.json()

            if self.interface['error']:
                raise basiciv.exceptions.AjaxRequestError(
                    self.interface['message']
                )

            if self.interface['body']:
                self.im_tag_url = self.interface['body']['thumbnail']
                self.im_name = self.im_tag_url.split('/')[-1]
                self.illust_id = self.im_name.split('.')[0] \
                    .replace('_p0_master1200', '') \
                    .replace('_p0', '')
            else:
                self.interface['body'] = {
                    'tag': unquote_plus(tag),
                    'abstract': None,
                    'thumbnail': None
                }

    def details(self):
        return self.interface['body']

    def view_tag(self):
        return self.interface['body']['tag']

    def view_abstract(self):
        return self.interface['body']['abstract']

    def view_thumbnail_image(self):
        if not self.im_data:
            headers = {
                'Referer': 'https://www.pixiv.net/member_illust.php?'
                           'mode=medium&illust_id=' + self.illust_id,
            }
            rg = self.sess.get(
                self.im_tag_url,
                headers=headers,
                timeout=2
            )
            self.im_data = rg.content

        im = Image.open(BytesIO(self.im_data))
        im.show()

    def save_tag_image(self) -> False:
        if not self.im_data:
            headers = {
                'Referer': 'https://www.pixiv.net/member_illust.php?'
                           'mode=medium&illust_id=' + self.illust_id,
            }
            rg = self.sess.get(
                self.im_tag_url,
                headers=headers,
                timeout=2)
            self.im_data = rg.content

        with open(self.im_name, 'wb') as f:
            f.write(self.im_data)
        return False



import re
from . import basiciv
from .helper import check_date
from . import worksiv


class Batch(basiciv.Queue):

    def __init__(self, **kwargs):
        super(Batch).__init__(**kwargs)
        self.que_tar = [
            {
                'illust_attrs': zip_[0],
                'rank': zip_[1],
                'yes_rank': zip_[2]
            } for zip_ in zip(
                kwargs['illust_attrs'],
                kwargs['rank'],
                kwargs['yes_rank']
            )
        ]


class Daily(basiciv.BasicConfig, basiciv.LoadInfo, basiciv.Queue):
    name = 'daily'
    rank_url = 'https://www.pixiv.net/ranking.php'
    rank_total = 0
    one_count = 0
    current_page = None
    current_date = None

    def __init__(self, ymd: int or str = None, filters: str = 'complex', **kwargs):
        """
        :param ymd:
            Example:
            ~~~~~~~
                20190101
                '2019-01-01'
                '2019/01/01'
                '2019.01.01'
        :param filters:
            Optional:
            ~~~~~~~~
                - complex
                - illust
                - ugoira (note: Unsupported view dynamic picture.)
                - manga
            default complex
        """
        super(Daily, self).__init__(
            **kwargs
        )

        self.params = {
            'mode': self.name,
            'date': '',
            'p': 1,
            'format': 'json'
        }

        if filters != 'complex':
            self.params.update({
                'content': filters,
            })

        if ymd:
            reg = re.compile(
                r'[0-9]{4}[^a-zA-Z0-9]?[0-9]{2}[^a-zA-Z0-9]?[0-9]{2}'
            )
            s = reg.fullmatch(str(ymd))
            date = s.string
            date = date.replace('/', '')\
                .replace('-', '')\
                .replace('.', '')

            year = int(date[:4])
            mouth = int(date[4:6])
            day = int(date[6:])

            check_date(year, mouth, day)

            self.params.update({
                'date': date,
            })
            self.current_date = date
            self.__run__(params=self.params)

    def __run__(self, params: {}):
        print(self.init_run)

        r = self.sess.get(
            self.rank_url,
            params=params,
            timeout=self.kvpair['timeout']
        )

        print(r.text)

        self.interface = r.json()

        if self.rank_total == 0:
            self.rank_total = self.interface['rank_total']
        self.current_page = self.interface['page']
        self.current_date = self.interface['date']

        print(self.init_finished)

        self.init_run = 'Current batch_size: {}, rank total: {}\n' \
                        'Loading date: {}, page: {} ...' \
            .format(
                len(self.que_tar), self.rank_total,
                params['date'], params['p']
            )
        self.init_finished = 'Load finished!'

    def run(self, ymd: int or str = None):
        if ymd:
            self.__init__(ymd=ymd)
        return self

    def one(self):
        if self.one_count < 50:
            curr_one = {
                'illust_attrs': worksiv.Works(
                    self.interface['contents'][self.one_count]['illust_id']
                ),
                'rank': self.interface['contents'][self.one_count]['rank'],
                'yes_rank': self.interface['contents'][self.one_count]['yes_rank']
            }
            self.one_count += 1
            self.que_tar.append(curr_one)
            return self.last()
        else:
            return self.next_page().one()

    def batch(self, nums: int = -1):
        if self.one_count < 50:
            list1, list2, list3 = [], [], []
            if nums == -1:
                while self.one_count < 50:
                    take = self.interface['contents'][self.one_count]

                    list1.append(worksiv.Works(take['illust_id']))
                    list2.append(take['rank'])
                    list3.append(take['yes_rank'])

                    self.one_count += 1
            else:
                for _ in range(nums):
                    if self.one_count == 50:
                        break
                    take = self.interface['contents'][self.one_count]

                    list1.append(worksiv.Works(take['illust_id']))
                    list2.append(take['rank'])
                    list3.append(take['yes_rank'])

                    self.one_count += 1

            curr_batch = Batch(
                illust_attrs=list1,
                rank=list2,
                yes_rank=list3
            )
            self.que_tar += curr_batch.que_tar
            return self.curr()
        else:
            return self.next_page().batch(nums)

    def prev_date(self):
        if self.interface['prev_date']:
            self.rank_total = 0
            self.step_number = 0
            self.one_count = 0
            self.que_tar.clear()
            self.params = {
                'date': self.interface['prev_date']
            }
            return self.run()

    def next_date(self):
        if self.interface['next_date']:
            self.rank_total = 0
            self.step_number = 0
            self.one_count = 0
            self.que_tar.clear()
            self.params.update({
                'date': self.interface['next_date']
            })
            return self.run()

    def prev_page(self):
        if self.interface['prev']:
            self.one_count = 0
            self.params.update({
                'p': self.interface['prev']
            })
            return self.run()

    def next_page(self):
        if self.interface['next']:
            self.one_count = 0
            self.params.update({
                'p': self.interface['next']
            })
            return self.run()


class Weekly(Daily):
    name = 'weekly'


class Monthly(Daily):
    name = 'monthly'


class Rookie(Daily):
    name = 'rookie'


class Original(Daily):
    name = 'original'


class Male(Daily):
    name = 'male'


class Female(Daily):
    name = 'female'


class DailyR(Daily):
    name = 'daily_r18'

    def __init__(self, ymd: int or str = None, filters: str = 'complex', **kwargs):
        super(DailyR, self).__init__(
            ymd=ymd,
            filters=filters,
            **kwargs
        )


class WeeklyR(DailyR):
    name = 'weekly_r18'


class MaleR(DailyR):
    name = 'male_r18'


class FemaleR(DailyR):
    name = 'female_r18'



from datetime import datetime
from .exceptions import DateError


def check_date(year: int, mouth: int, day: int) -> None:
    if year < 2008 or year > datetime.now().year:
        raise DateError(
            'No day {} in February {}'.format(day, year)
        )
    if mouth == 2:
        if year % 400 == 0:
            if day > 29 or day < 1:
                raise DateError(
                    'No day {} in February {}'.format(day, year)
                )
        else:
            if day > 28 or day < 1:
                raise DateError(
                    'No day {} in February {}'.format(day, year)
                )
    elif mouth == 1:
        if day > 31 or day < 1:
            raise DateError(
                'No day {} in January {}'.format(day, year)
            )
    elif mouth == 3:
        if day > 31 or day < 1:
            raise DateError(
                'No day {} in March {}'.format(day, year)
            )
    elif mouth == 4:
        if day > 30 or day < 1:
            raise DateError(
                'No day {} in April {}'.format(day, year)
            )
    elif mouth == 5:
        if day > 31 or day < 1:
            raise DateError(
                'No day {} in May {}'.format(day, year)
            )
    elif mouth == 6:
        if day > 30 or day < 1:
            raise DateError(
                'No day {} in June {}'.format(day, year)
            )
    elif mouth == 7:
        if day > 31 or day < 1:
            raise DateError(
                'No day {} in July {}'.format(day, year)
            )
    elif mouth == 8:
        if day > 31 or day < 1:
            raise DateError(
                'No day {} in August {}'.format(day, year)
            )
    elif mouth == 9:
        if day > 30 or day < 1:
            raise DateError(
                'No day {} in September {}'.format(day, year)
            )
    elif mouth == 10:
        if day > 31 or day < 1:
            raise DateError(
                'No day {} in October {}'.format(day, year)
            )
    elif mouth == 11:
        if day > 30 or day < 1:
            raise DateError(
                'No day {} in November {}'.format(day, year)
            )
    elif mouth == 12:
        if day > 31 or day < 1:
            raise DateError(
                'No day {} in December {}'.format(day, year)
            )
    else:
        raise DateError('Mouth error')



from . import basiciv
from . import useriv
from . import worksiv
from . import rankiv


class Pixiv(basiciv.Login):
    kind = ('daily', 'weekly', 'monthly', 'male', 'female', 'rookie', 'original',
            'daily_r18', 'weekly_r18', 'male_r18', 'female_r18')

    def works(self, illust_id: int or str) -> worksiv.Works:
        return worksiv.Works(illust_id, headers=self.sess.headers, token=self.token, user_id=self.user_id)

    def user(self, user_id: int or str) -> useriv.User:
        return useriv.User(user_id, headers=self.sess.headers, token=self.token, user_id=self.user_id)

    def rank(self, date, typed='daily', filters='complex') -> rankiv:
        if typed == 'weekly':
            return rankiv.Weekly(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'monthly':
            return rankiv.Monthly(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'male':
            return rankiv.Male(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'female':
            return rankiv.Female(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'rookie':
            return rankiv.Rookie(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'original':
            return rankiv.Original(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'daily_r18':
            return rankiv.DailyR(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'weekly_r18':
            return rankiv.WeeklyR(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'male_r18':
            return rankiv.MaleR(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        elif typed == 'female_r18':
            return rankiv.FemaleR(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )
        else:
            return rankiv.Daily(
                ymd=date,
                filters=filters,
                headers=self.sess.headers,
                token=self.token,
                user_id=self.user_id
            )



from requests.compat import quote_plus
from PIL import Image
from io import BytesIO
from . import basiciv
from . import tagiv
from . import useriv


class Discussion(basiciv.BasicConfig):

    def __init__(self, illust_id: int or str, comments_count, **kwargs):
        super(Discussion, self).__init__(**kwargs)
        self.url = 'https://www.pixiv.net/ajax/illusts/comments/roots?' \
                   'illust_id=%s&offset=0&limit=%s' % (
                       illust_id,
                       comments_count
                   )
        r = self.sess.get(
            self.url,
            timeout=5
        )
        self.interface = r.json()

        if self.interface['error']:
            raise basiciv.exceptions.AjaxRequestError(
                self.interface['message']
            )
        elif not self.interface['body']:
            self.interface['body'] = {'comments': None}

        self.que_tar = basiciv.Queue(self.interface['body']['comments'])


class Works(basiciv.BasicConfig):
    base_tags_que = None
    user_comments = None
    im_data = None

    def __init__(self, illust_id: int or str = None, **kwargs):
        super(Works, self).__init__(**kwargs)
        if illust_id:
            self.url = 'https://www.pixiv.net/ajax/illust/' \
                       '%s' % illust_id
            r = self.sess.get(
                self.url,
                headers={
                    'x-user-id': self.user_id
                },
                timeout=self.kvpair['timeout']
            )
            interface = r.json()

            if interface['error']:
                raise basiciv.exceptions.AjaxRequestError(
                    interface['message']
                )

            self.interface = interface['body']

    def author(self) -> useriv:
        return useriv.User(self.author_id())

    def details(self):
        return self.interface

    def create_date(self):
        return self.interface['createDate']

    def upload_date(self):
        return self.interface['uploadDate']

    def pages(self):
        return self.interface['pageCount']

    def illust_id(self):
        return self.interface['id']

    def imsize(self):
        return (self.interface['height'],
                self.interface['width'])

    def illust_title(self):
        return self.interface['title']

    def comment_count(self):
        return self.interface['commentCount']

    def mark_count(self):
        return self.interface['bookmarkCount']

    def like_count(self):
        return self.interface['likeCount']

    def view_tags(self) -> basiciv.Queue:
        if self.base_tags_que is None:
            tags_que = self.interface['tags']['tags']
            self.base_tags_que = basiciv.Queue([
                tagiv.WorksTag(quote_plus(tag['tag'])) for tag in tags_que])
        return self.base_tags_que

    def mini_url(self) -> tuple:
        count = self.pages()
        url = self.interface['urls']['mini']
        urls = ()
        for i in range(count):
            urls += (url.replace('_p0', '_p%s' % i),)
        return urls

    def thumb_url(self) -> tuple:
        count = self.pages()
        url = self.interface['urls']['thumb']
        urls = ()
        for i in range(count):
            urls += (url.replace('_p0', '_p%s' % i),)
        return urls

    def small_url(self) -> tuple:
        count = self.pages()
        url = self.interface['urls']['small']
        urls = ()
        for i in range(count):
            urls += (url.replace('_p0', '_p%s' % i),)
        return urls

    def regular_url(self) -> tuple:
        count = self.pages()
        url = self.interface['urls']['regular']
        urls = ()
        for i in range(count):
            urls += (url.replace('_p0', '_p%s' % i),)
        return urls

    def original_url(self) -> tuple:
        count = self.pages()
        url = self.interface['urls']['original']
        urls = ()
        for i in range(count):
            urls += (url.replace('_p0', '_p%s' % i),)
        return urls

    def author_name(self):
        return self.interface['userName']

    def author_id(self):
        return self.interface['userId']

    def view_count(self):
        return self.interface['viewCount']

    def view_comments(self) -> Discussion:
        if not self.user_comments:
            self.user_comments = Discussion(
                self.illust_id(),
                self.comment_count()
            )
        return self.user_comments

    def view_mini_image(self) -> None:
        print('Press key "s" save, non-"s" key page turn, key "Enter" determine, key "q" quit.')
        for i, illust_url in enumerate(self.mini_url()):
            self.sess.headers['Referer'] = 'https://www.pixiv.net/member_illust.php?' \
                           'mode=medium&illust_id=' + self.illust_id()
            rg = self.sess.get(illust_url, timeout=self.kvpair['timeout'])
            self.sess.headers.pop('Referer')
            self.im_data = rg.content
            im = Image.open(BytesIO(rg.content))
            im.show()
            gc = str(input())
            if gc == 's' or gc == 'S':
                if self.save(kind='mini', page_num=i):
                    gc = str(input('Continue to view? [Y] Yes | [Q] Quit'))
                    if gc == 'y' or gc == 'Y':
                        continue
            if gc == 'q' or gc == 'Q':
                break

    def view_thumb_image(self) -> None:
        print('Press key "s" save, non-"s" key page turn, key "Enter" determine, key "q" quit.')
        for i, illust_url in enumerate(self.thumb_url()):
            self.sess.headers['Referer'] = 'https://www.pixiv.net/member_illust.php?' \
                           'mode=medium&illust_id=' + self.illust_id()
            rg = self.sess.get(illust_url, timeout=self.kvpair['timeout'])
            self.sess.headers.pop('Referer')
            self.im_data = rg.content
            im = Image.open(BytesIO(rg.content))
            im.show()
            gc = str(input())
            if gc == 's' or gc == 'S':
                if self.save(kind='thumb', page_num=i):
                    gc = str(input('Continue to view? [Y] Yes | [Q] Quit'))
                    if gc == 'y' or gc == 'Y':
                        continue
            if gc == 'q' or gc == 'Q':
                break

    def view_small_image(self) -> None:
        print('Press key "s" save, non-"s" key page turn, key "Enter" determine, key "q" quit.')
        for i, illust_url in enumerate(self.small_url()):
            self.sess.headers['Referer'] = 'https://www.pixiv.net/member_illust.php?' \
                           'mode=medium&illust_id=' + self.illust_id()
            rg = self.sess.get(illust_url, timeout=self.kvpair['timeout'])
            self.sess.headers.pop('Referer')
            self.im_data = rg.content
            im = Image.open(BytesIO(rg.content))
            im.show()
            gc = str(input())
            if gc == 's' or gc == 'S':
                if self.save(kind='small', page_num=i):
                    gc = str(input('Continue to view? [Y] Yes | [Q] Quit'))
                    if gc == 'y' or gc == 'Y':
                        continue
            if gc == 'q' or gc == 'Q':
                break

    def view_regul_image(self) -> None:
        print('Press key "s" save, non-"s" key page turn, key "Enter" determine, key "q" quit.')
        for i, illust_url in enumerate(self.regular_url()):
            self.sess.headers['Referer'] = 'https://www.pixiv.net/member_illust.php?' \
                           'mode=medium&illust_id=' + self.illust_id()
            rg = self.sess.get(illust_url, timeout=10)
            self.sess.headers.pop('Referer')
            self.im_data = rg.content
            im = Image.open(BytesIO(rg.content))
            im.show()
            gc = str(input(':'))
            if gc == 's' or gc == 'S':
                if self.save(kind='regular', page_num=i):
                    gc = str(input('Continue to view? [Y] Yes | [Q] Quit\n:'))
                    if gc == 'y' or gc == 'Y':
                        continue
            if gc == 'q' or gc == 'Q':
                break

    def view_orig_image(self) -> None:
        print('Press key "s" save, non-"s" key page turn, key "Enter" determine, key "q" quit.')
        for i, illust_url in enumerate(self.original_url()):
            self.sess.headers['Referer'] = 'https://www.pixiv.net/member_illust.php?' \
                           'mode=medium&illust_id=' + self.illust_id()
            rg = self.sess.get(illust_url, timeout=self.kvpair['timeout'])
            self.sess.headers.pop('Referer')
            self.im_data = rg.content
            im = Image.open(BytesIO(rg.content))
            im.show()
            gc = str(input())
            if gc == 's' or gc == 'S':
                if self.save(kind='original', page_num=i):
                    gc = str(input('Continue to view? [Y] Yes | [Q] Quit'))
                    if gc == 'y' or gc == 'Y':
                        continue
            if gc == 'q' or gc == 'Q':
                break

    def save(self, kind: str = 'original', page_num: int = 1) -> True or None:
        """
        :param kind: [mini, thumb, small, regular, original]
        :param page_num: >=0 and < pages
        :return: returns True when param == valid and image == successfully downloaded, otherwise False.
        """
        if 0 <= page_num < self.pages():
            if kind == 'mini':
                illust_url = self.mini_url()[page_num]
            elif kind == 'thumb':
                illust_url = self.thumb_url()[page_num]
            elif kind == 'small':
                illust_url = self.small_url()[page_num]
            elif kind == 'regular':
                illust_url = self.regular_url()[page_num]
            else:
                illust_url = self.original_url()[page_num]

            im_name = illust_url.split('/')[-1]
            if not self.im_data:
                headers = {
                    'Referer': 'https://www.pixiv.net/member_illust.php?'
                               'mode=medium&illust_id=' + self.illust_id(),
                }
                rg = self.sess.get(
                    illust_url,
                    headers=headers,
                    timeout=self.kvpair['timeout']
                )
                self.im_data = rg.content
            try:
                with open(im_name, 'wb') as f:
                    f.write(self.im_data)
            except IOError:
                raise basiciv.exceptions.PixivIoError('Save original image failed!')
            return True
        else:
            raise basiciv.exceptions.PixivError('Page code error!')

    def like(self) -> True or None:
        if 'cookies' not in self.sess.headers:
            raise basiciv.exceptions.PixivError('You must be logged in before using this feature!')

        self.sess.headers.update({
            'referer': 'https://www.pixiv.net/member_illust.php?'
                       'mode=medium&illust_id=' + self.author_id(),
            'sec-fetch-mode': 'cors',
            'sec-fetch-site': 'same-origin',
            'x-csrf-token': self.token,
        })
        interface = self.sess.post(
            'https://www.pixiv.net/ajax/illusts/like',
            json={'illust_id': self.illust_id()}).json()
        self.sess.headers.pop('x-csrf-token')
        self.sess.headers.pop('referer')
        if not interface['error']:
            return True
        else:
            raise basiciv.exceptions.AutheVerifyError(interface['message'])

    def mark(self) -> True or None:
        if 'cookies' not in self.sess.headers:
            raise basiciv.exceptions.PixivError('You must be logged in before using this feature!')

        interface = self.sess.post(
            'https://www.pixiv.net/ajax/illusts/bookmarks/add',
            json={
                'comment': '',
                'illust_id': self.illust_id(),
                'restrict': 0,
                'tags': []
            },
            headers={
                'x-csrf-token': self.token
            }
        ).json()
        if not interface['error']:
            return True
        else:
            raise basiciv.exceptions.AutheVerifyError(interface['message'])

    def bookmark(self) -> True or None:
        if 'cookies' not in self.sess.headers:
            raise basiciv.exceptions.PixivError('You must be logged in before using this feature!')

        interface = self.sess.post(
            'https://www.pixiv.net/bookmark_add.php',
            data={
                'mode': 'add',
                'type': 'user',
                'user_id': self.author_id(),
                'tags': '',
                'restrict': 0,
                'format': 'json',
            },
            headers={
                'x-csrf-token': self.token
            }
        ).json()
        if not interface['error']:
            return True
        else:
            raise basiciv.exceptions.AutheVerifyError(interface['message'])



class PixivError(Exception):
    """ Basic pixiv exception. """


class DateError(PixivError):
    """ Date range exception. """


class AjaxRequestError(PixivError):
    """ Some unexpected errors may occur. """


class AutheVerifyError(PixivError):
    """ Cookie or token verification error. """


class PixivIoError(PixivError):
    """ IO write/read Exception. """



import re
import requests
from requests import Session
from . import exceptions
import json


class BasicConfig(object):
    sess = Session()
    token = None
    user_id = None
    user_agent = 'Mozilla/5.0 (Windows NT 6.1; Win64; x64)' \
                 ' AppleWebKit/537.36 (KHTML, like Gecko)' \
                 ' Chrome/71.0.3578.98 Safari/537.36'
    kvpair = {
        'cookie': False,
        'proxy': '',
        'save_cookies': False,
        'timeout': 8
    }

    def __init__(self, headers: {} = None, token: str = None, user_id: int or str = None):
        if headers:
            self.sess.headers.update(headers)
        else:
            self.sess.headers.update({
                'accept-encoding': 'gzip, deflate, br',
                'user-agent': self.user_agent,
                'sec-fetch-mode': 'cors',
                'sec-fetch-site': 'same-origin'
            })

        if token:
            self.token = token

        if user_id:
            self.user_id = user_id


class Login(BasicConfig):
    init_url = 'https://www.pixiv.net/'
    login_api = 'https://accounts.pixiv.net/api/login'
    
    username = None
    password = None

    def __init__(self, username: str = None, password: str = None, **kwargs):
        super(Login, self).__init__()
        self.kvpair.update(kwargs)

        if self.kvpair['cookie']:
            cookie_json = json.load(open('../cookies'))
            self.sess.cookies = requests.utils.cookiejar_from_dict(cookie_json)
            self.token = self.get_postkey()

        elif username and password:
            self.username = username
            self.password = password

            self.token = self.get_postkey()

            data = {
                'pixiv_id': self.username,
                'captcha': '',
                'g_recaptcha_response': '',
                'password': self.password,
                'post_key': self.token,
                'source': 'pc',
                'ref': 'wwwtop_accounts_index',
                'return_to': self.init_url
            }

            login = self.sess.post(self.login_api, data=data, timeout=self.kvpair['timeout'])

            log_msg = login.json()

            if log_msg['error']:
                raise exceptions.PixivError(log_msg['message'])

            if 'validation_errors' not in log_msg['body']:
                print('Login successful!')
            elif 'pixiv_id' in log_msg['body']['validation_errors']:
                raise exceptions.PixivError(
                    log_msg['body']['validation_errors']['pixiv_id']
                )
            elif 'captcha' in log_msg['body']['validation_errors']:
                raise exceptions.PixivError(
                    log_msg['body']['validation_errors']['captcha']
                )

            if self.kvpair['save_cookies']:
                cookie_json = requests.utils.dict_from_cookiejar(self.sess.cookies)
                json.dump(cookie_json, open('../cookies', 'w+'),)

            cookie = "; ".join([str(x) + "=" + str(y) for x, y in self.sess.cookies.items()])
            headers = {
                'cookie': cookie,
            }
            self.sess.headers.update(headers)

    def login(self, username: str, password: str):
        self.username = username
        self.password = password
        self.__init__(username=username, password=password)

    def get_postkey(self):
        r = self.sess.get(self.init_url, timeout=5)
        reg = re.compile(r'.*pixiv.context.token = "([a-z0-9]{32})"?.*')
        return reg.findall(r.text)[0]


class LoadInfo:
    init_run = 'Pixiver Initializing...'
    init_finished = 'Initialized!'


class Queue(object):
    step_number = 0
    que_tar = []

    def __init__(self, argv: [] = None):
        if argv:
            self.que_tar = argv

    def first(self):
        return self.que_tar[0]

    def curr(self):
        return self.que_tar[self.step_number]

    def prev(self):
        if self.step_number < 0:
            raise Exception('Lower limit.')
        self.step_number -= 1
        return self.que_tar[self.step_number]

    def next(self):
        if self.step_number > len(self.que_tar) - 1:
            raise Exception('Upper limit.')
        self.step_number += 1
        return self.que_tar[self.step_number]

    def last(self):
        return self.que_tar[len(self.que_tar) - 1]

    def size(self):
        return len(self.que_tar)

    def remove(self):
        self.que_tar.pop(self.step_number)



from setuptools import setup, find_packages

with open("README-cn.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

setup(
    name="pixiver",
    version="0.0.9.1252",
    author="darkchii",
    author_email="darkchii@qq.com",
    description="This is a python package for get illustration on the pixiv by ajax interfaces.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['requests', 'pillow'],
    packages=find_packages(),
    url='https://github.com/darkchii/pixiver',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet'
    ]
)
